from Products.Five import zcml
from Products.Five import fiveconfigure

from Testing import ZopeTestCase as ztc

from Products.PloneTestCase import PloneTestCase as ptc
from Products.PloneTestCase.layer import onsetup
from Products.Five.testbrowser import Browser
from zope.component import getUtility, getMultiAdapter
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletAssignmentMapping
from Products.PloneTestCase.setup import portal_owner, default_password


@onsetup
def setup_fui_memberlist():
	"""Set up the additional products required for the fui.memberlist product.
	
	The @onsetup decorator causes the execution of this body to be deferred
	until the setup of the Plone site testing layer.
	"""
	
	# Load the ZCML configuration for the optilux.policy package.
	# This includes the other products below as well.
	
	fiveconfigure.debug_mode = True
	import fui.memberlist
	zcml.load_config('configure.zcml', fui.memberlist)
	fiveconfigure.debug_mode = False
	
	# We need to tell the testing framework that these products
	# should be available. This can't happen until after we have loaded
	# the ZCML.
	
	ztc.installPackage('fui.memberlist')
	
# The order here is important: We first call the (deferred) function which
# installs the products we need for the Optilux package. Then, we let 
# PloneTestCase set up this product on installation.

setup_fui_memberlist()
ptc.setupPloneSite(products=['fui.memberlist'])

class FuiMemberListTestCase(ptc.PloneTestCase):
	"""Base class used for test cases. """
		
class FuiMemberListFunctionalTestCase(ptc.FunctionalTestCase):
	"""Test case class used for functional (doc-)tests """

	def afterSetUp(self):
		self.browser = Browser()

		# The following is useful when writing and debugging testself.browser tests. It lets
		# us see error messages properly.
		self.browser.handleErrors = False
		self.portal.error_log._ignored_exceptions = ()

		# We then turn off the various portlets, because they sometimes duplicate links
		# and text (e.g. the navtree, the recent recent items listing) that we wish to
		# test for in our own views. Having no portlets makes things easier.
		left_column = getUtility(IPortletManager, name=u"plone.leftcolumn")
		left_assignable = getMultiAdapter((self.portal, left_column), IPortletAssignmentMapping)
		for name in left_assignable.keys():
			del left_assignable[name]
		right_column = getUtility(IPortletManager, name=u"plone.rightcolumn")
		right_assignable = getMultiAdapter((self.portal, right_column), IPortletAssignmentMapping)
		for name in right_assignable.keys():
			del right_assignable[name]


	def loginAdminClick(self):
		portal_url = self.portal.absolute_url()
		self.browser.open(portal_url + '/login_form?came_from=' + portal_url)
		self.browser.getControl(name='__ac_name').value = portal_owner
		self.browser.getControl(name='__ac_password').value = default_password
		self.browser.getControl(name='submit').click()

	def logoutClick(self):
		portal_url = self.portal.absolute_url()
		self.browser.getLink("Log out").click()



from zope.interface import Interface
from zope import schema

from zope.app.container.constraints import contains

from fui.memberlist import MemberListMessageFactory as _


class IMemberList(Interface):
	""" A memberlist. """



"""Definition of the MemberList content type and associated schemata and
other logic.

This file contains a number of comments explaining the various lines of
code. Other files in this sub-package contain analogous code, but will 
not be commented as heavily.

Please see README.txt for more information on how the content types in
this package are used.
"""

import re

from zope.interface import implements
from zope.component import adapter, getMultiAdapter, getUtility
from zope.app.container.interfaces import INameChooser
from Acquisition import aq_inner, aq_parent
from Products.Archetypes import atapi
from Products.Archetypes.interfaces import IObjectInitializedEvent
from Products.ATContentTypes.content import schemata
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATContentTypes.content import base

from fui.memberlist.interfaces import IMemberList
from fui.memberlist.config import PROJECTNAME
from fui.memberlist import MemberListMessageFactory as _



# This is the Archetypes schema, defining fields and widgets. We extend
# the one from ATContentType's ATFolder with our additional fields.
schema = schemata.ATContentTypeSchema.copy() + atapi.Schema((
	atapi.LinesField("currentmembers",
		required = True,
		searchable = False,
		storage = atapi.AnnotationStorage(),
		widget = atapi.LinesWidget(
				label = u"Current FUI members.",
				description = "One username on each line.")
		),
	atapi.LinesField("exclude",
		required = True,
		searchable = False,
		storage = atapi.AnnotationStorage(),
		widget = atapi.LinesWidget(
				label = u"Exclude.",
				description = "Users which are excluded from this listing. "\
						"One username on each line.")
		),
	))

# We want to ensure that the properties we use as field properties (see
# below), use AnnotationStorage. Without this, our property will conflict
# with the attribute with the same name that is being managed by the default
# attributestorage
schema['title'].storage = atapi.AnnotationStorage()


class MemberList(base.ATCTContent):
	""" """
	implements(IMemberList)
	
	# The portal type name must be set here, matching the one in types.xml
	# in the GenericSetup profile
	portal_type = "MemberList"
	
	# We then associate the schema with our content type
	schema = schema 


# This line tells Archetypes about the content type
atapi.registerType(MemberList, PROJECTNAME)



import unittest

from zope.testing import doctestunit, doctest
from zope.component import testing, eventtesting

from Testing import ZopeTestCase as ztc

from fui.memberlist.tests import base


optionflags = doctest.REPORT_ONLY_FIRST_FAILURE | \
		doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS

def test_suite():
    return unittest.TestSuite([
		ztc.ZopeDocFileSuite(
			'tests/functional.doctest',
			package = 'fui.memberlist',
			test_class = base.FuiMemberListFunctionalTestCase,
			optionflags = optionflags),
		])

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



from os import listdir, sep
from os.path import isdir, join, exists, dirname
import codecs

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from Products.statusmessages.interfaces import IStatusMessage




class MemberList(BrowserView):
	__call__ = ViewPageTemplateFile('memberlist.pt')

	def __init__(self, *args, **kwargs):
		BrowserView.__init__(self, *args, **kwargs)
		self.ctx = aq_inner(self.context)
		self.exclude = set(self.ctx.getExclude())
		self.current = set(self.ctx.getCurrentmembers())

	def currentMembers(self):
		pm = self.ctx.portal_membership
		out = []
		for memberId in self.current:
			member = pm.getMemberById(memberId)
			if member:
				out.append(self._memberInfo(memberId, member))
		return out

	def previousMembers(self):
		pm = self.ctx.portal_membership
		out = []
		for memberId in pm.listMemberIds():
			if not memberId in self.exclude and not memberId in self.current:
				member = pm.getMemberById(memberId)
				out.append(self._memberInfo(memberId, member))
		return out

	def _memberInfo(self, memberId, member):
		portal = getToolByName(self.ctx, "portal_url").getPortalObject()
		portrait = portal.portal_memberdata.portraits.get("espeak")
		if portrait:
			portrait = portrait.absolute_url()
		else:
			portrait = "defaultUser.gif"
		homeFolder = "%s/%s" % (portal.Members.absolute_url(), memberId)

		return dict(
				email = member.getProperty("email"),
				fullname = member.getProperty("fullname"),
				portrait_url = member.getPersonalPortrait().absolute_url(),
				homeFolder = homeFolder,
				description = member.getProperty("description")
			)
